# -*- coding: utf-8 -*-
from django.utils.encoding import force_unicode

import json
import sys
from django.conf import urls
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _


def uniopen(filename):
    return (line.decode().rstrip('\n') for line in open(filename))


def uniprint(text, file=sys.stdout):
    print >>file, text.encode('utf-8')


def debug(entry, text):
    print>> sys.stderr, (u'%s: %s' % (entry, text)).encode()


def error_redirect(request, error, redirect_url='/'):
    request.session['error'] = error
    return HttpResponseRedirect(redirect_url)


def message_redirect(request, message, redirect_url='/'):
    request.session['message'] = message
    return HttpResponseRedirect(redirect_url)


def make_form(request, form_class, **kwargs):
    if request.POST.get('det', '') == form_class.base_fields['det'].initial:
        return form_class(data=request.POST, files=request.FILES, **kwargs)
    else:
        return form_class(**kwargs)


def invert(l):
    return dict((e, nr) for (nr, e) in enumerate(l))


def generator_slice(generator, count):
    res = []
    try:
        for i in range(count):
            res.append(generator.next())
    except StopIteration:
        pass
    return res


def url(regex, view, **kwargs):
    if 'name' not in kwargs:
        kwargs['name'] = view.rsplit('.', 1)[-1]
    return urls.url(regex, view, **kwargs)


def stringify_keys(dictionary):
    return dict((keyword.encode('ascii'), value)
                for keyword, value in dictionary.iteritems())


# copypasta ze standardowego modułu bisect
def bisect_left(a, x, lo=0, hi=None, cmp=None):
    if cmp is None:
        cmp = __builtins__.cmp
    if lo < 0:
        raise ValueError('lo must be non-negative')
    if hi is None:
        hi = len(a)
    while lo < hi:
        mid = (lo + hi) // 2
        if cmp(a[mid], x) < 0:
            lo = mid + 1
        else:
            hi = mid
    return lo


def no_history():
    from django.db import connection

    cursor = connection.cursor()
    cursor.execute("SELECT set_config('var.user_id', '0', false)")


def reverse(seq):
    return seq[::-1]


def flatten(seq):
    return [item for subseq in seq for item in subseq]


def suffix(string, length):
    return string[-length:] if length > 0 else ''


def suffixes(s):
    return [s[i:] for i in range(len(s) + 1)]


def cut_end(s, end):
    assert s.endswith(end)
    n = len(end)
    if n == 0:
        return s
    else:
        return s[:-n]


def error_messages(form):
    return '\n'.join(
        '%s: %s' % (form.fields[k].label, ' '.join(v)) if k != '__all__'
        else ' '.join(v) for k, v in form.errors.iteritems())


class GroupDict(dict):
    def add(self, key, value):
        if key not in self:
            self[key] = []
        self[key].append(value)


def json_encode(obj, sort_keys=True, ensure_ascii=False):
    return json.dumps(obj, sort_keys=sort_keys, ensure_ascii=ensure_ascii)


def json_decode(obj):
    return json.loads(obj)


class FakeQueryset(list):
    def all(self):
        return self


def format_date(date):
    return date.strftime(force_unicode(_('%d.%m.%Y %H:%M')))


def format_date_exact(date):
    return date.strftime(force_unicode(_('%d.%m.%Y %H:%M:%S.%f')))
