# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):
    def forwards(self, orm):
        # Adding model 'QualifierExclusionClass'
        db.create_table('klasy_wykluczania', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True,
                max_length=64, db_column='nazwa')),
            ('vocabulary',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.Vocabulary'], db_column='slownik')),
        ))
        db.send_create_signal('dictionary', ['QualifierExclusionClass'])

        # Adding model 'Qualifier'
        db.create_table('kwalifikatory', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('label',
            self.gf('django.db.models.fields.CharField')(max_length=64,
                db_column='kwal')),
            ('vocabulary',
            self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='qualifiers', db_column='slownik',
                to=orm['dictionary.Vocabulary'])),
            ('exclusion_class',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.QualifierExclusionClass'], null=True,
                db_column='klasa', blank=True)),
            ('deleted',
            self.gf('django.db.models.fields.BooleanField')(default=False,
                db_column='usuniety')),
        ))
        db.send_create_signal('dictionary', ['Qualifier'])

        # Adding unique constraint on 'Qualifier', fields ['label', 'vocabulary']
        db.create_unique('kwalifikatory', ['kwal', 'slownik'])

        # Adding model 'Classification'
        db.create_table('klasyfikacje', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True,
                max_length=64, db_column='nazwa')),
        ))
        db.send_create_signal('dictionary', ['Classification'])

        # Adding model 'ClassificationValue'
        db.create_table('wartosci_klasyfikacji', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('label', self.gf('django.db.models.fields.CharField')(unique=True,
                max_length=64, db_column='nazwa')),
            ('classification',
            self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='values', db_column='klas_id',
                to=orm['dictionary.Classification'])),
            ('parent_node',
            self.gf('django.db.models.fields.related.ForeignKey')(blank=True,
                related_name='child_nodes', null=True, db_column='rodzic',
                to=orm['dictionary.ClassificationValue'])),
            ('deleted',
            self.gf('django.db.models.fields.BooleanField')(default=False,
                db_column='usunieta')),
        ))
        db.send_create_signal('dictionary', ['ClassificationValue'])

        # Adding M2M table for field lexemes on 'ClassificationValue'
        db.create_table('wartosci_klasyfikacji_lexemes', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True,
                auto_created=True)),
            ('classificationvalue',
            models.ForeignKey(orm['dictionary.classificationvalue'],
                null=False)),
            ('lexeme', models.ForeignKey(orm['dictionary.lexeme'], null=False))
        ))
        db.create_unique('wartosci_klasyfikacji_lexemes',
            ['classificationvalue_id', 'lexeme_id'])

        # Adding model 'LexicalClass'
        db.create_table('czescimowy', (
            ('symbol',
            self.gf('django.db.models.fields.CharField')(max_length=16,
                primary_key=True, db_column='czm')),
        ))
        db.send_create_signal('dictionary', ['LexicalClass'])

        # Adding model 'PartOfSpeech'
        db.create_table('klasygramatyczne', (
            ('symbol',
            self.gf('django.db.models.fields.CharField')(max_length=16,
                primary_key=True, db_column='pos')),
            ('lexical_class',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.LexicalClass'], db_column='czm')),
        ))
        db.send_create_signal('dictionary', ['PartOfSpeech'])

        # Adding model 'BaseFormLabel'
        db.create_table('efobazy', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('entry',
            self.gf('django.db.models.fields.CharField')(max_length=32,
                db_column='efobaz', blank=True)),
        ))
        db.send_create_signal('dictionary', ['BaseFormLabel'])

        # Adding model 'InflectionCharacteristic'
        db.create_table('charfle', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('entry',
            self.gf('django.db.models.fields.CharField')(max_length=16,
                db_column='charfl', blank=True)),
            ('part_of_speech',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.PartOfSpeech'], db_column='pos')),
            ('basic_form_label',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.BaseFormLabel'], db_column='efobaz')),
        ))
        db.send_create_signal('dictionary', ['InflectionCharacteristic'])

        # Adding unique constraint on 'InflectionCharacteristic', fields ['entry', 'part_of_speech']
        db.create_unique('charfle', ['charfl', 'pos'])

        # Adding model 'PatternType'
        db.create_table('typywzorow', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('lexical_class',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.LexicalClass'], db_column='czm')),
            ('entry',
            self.gf('django.db.models.fields.CharField')(max_length=32,
                db_column='wtyp', blank=True)),
        ))
        db.send_create_signal('dictionary', ['PatternType'])

        # Adding model 'Pattern'
        db.create_table('wzory', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True,
                max_length=32, db_column='w_id')),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.PatternType'], db_column='typ')),
            ('example',
            self.gf('django.db.models.fields.CharField')(max_length=64,
                db_column='przyklad')),
            ('basic_form_ending',
            self.gf('django.db.models.fields.CharField')(max_length=32,
                db_column='zakp', blank=True)),
            ('status',
            self.gf('django.db.models.fields.CharField')(max_length=8)),
            ('comment',
            self.gf('django.db.models.fields.TextField')(db_column='komentarz',
                blank=True)),
        ))
        db.send_create_signal('dictionary', ['Pattern'])

        # Adding model 'Ending'
        db.create_table('zakonczenia', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('pattern', self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='endings', db_column='w_id',
                to=orm['dictionary.Pattern'])),
            ('base_form_label',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.BaseFormLabel'], db_column='efobaz')),
            ('index',
            self.gf('django.db.models.fields.IntegerField')(db_column='zind')),
            ('string',
            self.gf('django.db.models.fields.CharField')(max_length=16,
                db_column='zak', blank=True)),
        ))
        db.send_create_signal('dictionary', ['Ending'])

        # Adding unique constraint on 'Ending', fields ['pattern', 'base_form_label', 'index']
        db.create_unique('zakonczenia', ['w_id', 'efobaz', 'zind'])

        # Adding M2M table for field qualifiers on 'Ending'
        db.create_table('kwalifikatory_zakonczen', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True,
                auto_created=True)),
            ('ending', models.ForeignKey(orm['dictionary.ending'], null=False)),
            ('qualifier',
            models.ForeignKey(orm['dictionary.qualifier'], null=False))
        ))
        db.create_unique('kwalifikatory_zakonczen',
            ['ending_id', 'qualifier_id'])

        # Adding model 'Lexeme'
        db.create_table('leksemy', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('entry',
            self.gf('django.db.models.fields.CharField')(db_index=True,
                max_length=64, db_column='haslo', blank=True)),
            ('entry_suffix',
            self.gf('django.db.models.fields.CharField')(max_length=16,
                db_column='haslosuf', blank=True)),
            ('gloss',
            self.gf('django.db.models.fields.TextField')(db_column='glosa',
                blank=True)),
            ('homonym_number',
            self.gf('django.db.models.fields.IntegerField')(db_column='hom')),
            ('part_of_speech',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.PartOfSpeech'], db_column='pos')),
            ('owner_vocabulary',
            self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='owned_lexemes', db_column='slownik',
                to=orm['dictionary.Vocabulary'])),
            ('source',
            self.gf('django.db.models.fields.CharField')(max_length=32,
                db_column='zrodlo', blank=True)),
            ('status',
            self.gf('django.db.models.fields.CharField')(max_length=8,
                db_column='status')),
            ('comment',
            self.gf('django.db.models.fields.TextField')(db_column='komentarz',
                blank=True)),
            ('last_modified',
            self.gf('django.db.models.fields.DateTimeField')(auto_now=True,
                db_column='data_modyfikacji', blank=True)),
            ('responsible',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['auth.User'], null=True, db_column='odpowiedzialny',
                blank=True)),
            ('deleted',
            self.gf('django.db.models.fields.BooleanField')(default=False,
                db_column='usuniety')),
        ))
        db.send_create_signal('dictionary', ['Lexeme'])

        # Adding M2M table for field qualifiers on 'Lexeme'
        db.create_table('kwalifikatory_leksemow', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True,
                auto_created=True)),
            ('lexeme', models.ForeignKey(orm['dictionary.lexeme'], null=False)),
            ('qualifier',
            models.ForeignKey(orm['dictionary.qualifier'], null=False))
        ))
        db.create_unique('kwalifikatory_leksemow',
            ['lexeme_id', 'qualifier_id'])

        # Adding model 'LexemeInflectionPattern'
        db.create_table('odmieniasie', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('lexeme', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.Lexeme'], db_column='l_id')),
            ('index',
            self.gf('django.db.models.fields.IntegerField')(db_column='oind')),
            ('pattern', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.Pattern'], db_column='w_id')),
            ('inflection_characteristic',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.InflectionCharacteristic'],
                db_column='charfl')),
            ('root', self.gf('django.db.models.fields.CharField')(max_length=64,
                db_column='rdzen')),
        ))
        db.send_create_signal('dictionary', ['LexemeInflectionPattern'])

        # Adding unique constraint on 'LexemeInflectionPattern', fields ['lexeme', 'index']
        db.create_unique('odmieniasie', ['l_id', 'oind'])

        # Adding M2M table for field qualifiers on 'LexemeInflectionPattern'
        db.create_table('kwalifikatory_odmieniasiow', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True,
                auto_created=True)),
            ('lexemeinflectionpattern',
            models.ForeignKey(orm['dictionary.lexemeinflectionpattern'],
                null=False)),
            ('qualifier',
            models.ForeignKey(orm['dictionary.qualifier'], null=False))
        ))
        db.create_unique('kwalifikatory_odmieniasiow',
            ['lexemeinflectionpattern_id', 'qualifier_id'])

        # Adding model 'Vocabulary'
        db.create_table('slowniki', (
            ('id', self.gf('django.db.models.fields.CharField')(max_length=64,
                primary_key=True, db_column='slownik')),
        ))
        db.send_create_signal('dictionary', ['Vocabulary'])

        # Adding M2M table for field managers on 'Vocabulary'
        db.create_table('slowniki_managers', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True,
                auto_created=True)),
            ('vocabulary',
            models.ForeignKey(orm['dictionary.vocabulary'], null=False)),
            ('user', models.ForeignKey(orm['auth.user'], null=False))
        ))
        db.create_unique('slowniki_managers', ['vocabulary_id', 'user_id'])

        # Adding M2M table for field viewers on 'Vocabulary'
        db.create_table('slowniki_viewers', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True,
                auto_created=True)),
            ('vocabulary',
            models.ForeignKey(orm['dictionary.vocabulary'], null=False)),
            ('user', models.ForeignKey(orm['auth.user'], null=False))
        ))
        db.create_unique('slowniki_viewers', ['vocabulary_id', 'user_id'])

        # Adding M2M table for field editors on 'Vocabulary'
        db.create_table('slowniki_editors', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True,
                auto_created=True)),
            ('vocabulary',
            models.ForeignKey(orm['dictionary.vocabulary'], null=False)),
            ('user', models.ForeignKey(orm['auth.user'], null=False))
        ))
        db.create_unique('slowniki_editors', ['vocabulary_id', 'user_id'])

        # Adding M2M table for field classifications on 'Vocabulary'
        db.create_table('slowniki_classifications', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True,
                auto_created=True)),
            ('vocabulary',
            models.ForeignKey(orm['dictionary.vocabulary'], null=False)),
            ('classification',
            models.ForeignKey(orm['dictionary.classification'], null=False))
        ))
        db.create_unique('slowniki_classifications',
            ['vocabulary_id', 'classification_id'])

        # Adding model 'LexemeAssociation'
        db.create_table('leksemy_w_slownikach', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('lexeme', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.Lexeme'], db_column='l_id')),
            ('vocabulary',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.Vocabulary'], db_column='slownik')),
        ))
        db.send_create_signal('dictionary', ['LexemeAssociation'])

        # Adding model 'CrossReferenceType'
        db.create_table('typyodsylaczy', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('symbol',
            self.gf('django.db.models.fields.CharField')(max_length=10,
                db_column='typods')),
            ('desc', self.gf('django.db.models.fields.CharField')(max_length=40,
                db_column='naglowek')),
            ('index', self.gf('django.db.models.fields.IntegerField')(
                db_column='kolejnosc')),
            ('from_pos', self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='crtype_to', db_column='pos1',
                to=orm['dictionary.PartOfSpeech'])),
            ('to_pos', self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='crtype_from', db_column='pos2',
                to=orm['dictionary.PartOfSpeech'])),
        ))
        db.send_create_signal('dictionary', ['CrossReferenceType'])

        # Adding model 'CrossReference'
        db.create_table('odsylacze', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('from_lexeme',
            self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='refs_to', db_column='l_id_od',
                to=orm['dictionary.Lexeme'])),
            ('to_lexeme', self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='refs_from', db_column='l_id_do',
                to=orm['dictionary.Lexeme'])),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.CrossReferenceType'],
                db_column='typods_id')),
        ))
        db.send_create_signal('dictionary', ['CrossReference'])

        # Adding model 'Variant'
        db.create_table('warianty', (
            ('id', self.gf('django.db.models.fields.CharField')(max_length=32,
                primary_key=True, db_column='wariant')),
        ))
        db.send_create_signal('dictionary', ['Variant'])

        # Adding model 'TableTemplate'
        db.create_table('szablony_tabel', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('variant', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.Variant'], db_column='wariant')),
            ('pattern_type',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.PatternType'], db_column='wtyp')),
            ('inflection_characteristic',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.InflectionCharacteristic'],
                db_column='charfl')),
        ))
        db.send_create_signal('dictionary', ['TableTemplate'])

        # Adding model 'Cell'
        db.create_table('klatki', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('table_template',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.TableTemplate'], db_column='st_id')),
            ('base_form_label',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.BaseFormLabel'], db_column='efobaz')),
            ('tag',
            self.gf('django.db.models.fields.TextField')(db_column='tag',
                blank=True)),
            ('prefix',
            self.gf('django.db.models.fields.CharField')(max_length=20,
                db_column='prefiks', blank=True)),
            ('suffix',
            self.gf('django.db.models.fields.CharField')(max_length=20,
                db_column='sufiks', blank=True)),
            ('index',
            self.gf('django.db.models.fields.IntegerField')(db_column='kind')),
        ))
        db.send_create_signal('dictionary', ['Cell'])

        # Adding model 'TableCell'
        db.create_table('komorki_tabel', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('cell', self.gf('django.db.models.fields.related.OneToOneField')(
                to=orm['dictionary.Cell'], unique=True, db_column='k_id')),
            ('row', self.gf('django.db.models.fields.IntegerField')()),
            ('col', self.gf('django.db.models.fields.IntegerField')()),
            ('rowspan', self.gf('django.db.models.fields.IntegerField')()),
            ('colspan', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('dictionary', ['TableCell'])

        # Adding model 'TableHeader'
        db.create_table('naglowki_tabel', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('table_template',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.TableTemplate'], db_column='st_id')),
            ('row', self.gf('django.db.models.fields.IntegerField')()),
            ('col', self.gf('django.db.models.fields.IntegerField')()),
            ('rowspan', self.gf('django.db.models.fields.IntegerField')()),
            ('colspan', self.gf('django.db.models.fields.IntegerField')()),
            ('label',
            self.gf('django.db.models.fields.CharField')(max_length=64,
                db_column='nagl', blank=True)),
            ('row_header',
            self.gf('django.db.models.fields.BooleanField')(default=False,
                db_column='wierszowy')),
        ))
        db.send_create_signal('dictionary', ['TableHeader'])

        # Adding model 'ParadygmatyWSJP'
        db.create_table('paradygmatywsjp', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('wariant',
            self.gf('django.db.models.fields.CharField')(max_length=4)),
            ('typr', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.PatternType'], db_column='typr')),
            ('charfl', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.InflectionCharacteristic'],
                db_column='charfl')),
            ('podparad',
            self.gf('django.db.models.fields.CharField')(max_length=4,
                blank=True)),
            ('row', self.gf('django.db.models.fields.IntegerField')()),
            ('col', self.gf('django.db.models.fields.IntegerField')()),
            ('rowspan', self.gf('django.db.models.fields.IntegerField')()),
            ('colspan', self.gf('django.db.models.fields.IntegerField')()),
            ('efobaz', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.BaseFormLabel'], db_column='efobaz')),
            ('morf', self.gf('django.db.models.fields.TextField')()),
            ('pref', self.gf('django.db.models.fields.CharField')(max_length=20,
                blank=True)),
            ('suf', self.gf('django.db.models.fields.CharField')(max_length=20,
                blank=True)),
            ('kskl', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('dictionary', ['ParadygmatyWSJP'])

        # Adding model 'LexemeForm'
        db.create_table('dictionary_lexemeform', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('lexeme', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.Lexeme'])),
            ('form',
            self.gf('django.db.models.fields.CharField')(max_length=128,
                db_index=True)),
        ))
        db.send_create_signal('dictionary', ['LexemeForm'])

        # Adding model 'SavedFilter'
        db.create_table('dictionary_savedfilter', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('serialized_filter',
            self.gf('django.db.models.fields.TextField')()),
            ('name',
            self.gf('django.db.models.fields.CharField')(max_length=64)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['auth.User'])),
            ('super',
            self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('dictionary', ['SavedFilter'])

        # Adding unique constraint on 'SavedFilter', fields ['name', 'user']
        db.create_unique('dictionary_savedfilter', ['name', 'user_id'])

        # Adding model 'SavedExportData'
        db.create_table('dictionary_savedexportdata', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('serialized_data', self.gf('django.db.models.fields.TextField')()),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True,
                max_length=64)),
        ))
        db.send_create_signal('dictionary', ['SavedExportData'])

        # Adding model 'History'
        db.create_table('history', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('table_name',
            self.gf('django.db.models.fields.CharField')(max_length=120,
                db_column='table_name_')),
            ('column_name',
            self.gf('django.db.models.fields.CharField')(max_length=120,
                db_column='column_name_', blank=True)),
            ('timestamp', self.gf('django.db.models.fields.DateTimeField')(
                db_column='timestamp_')),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['auth.User'], db_column='user_id_')),
            ('old_value',
            self.gf('django.db.models.fields.TextField')(db_column='old_value_',
                blank=True)),
            ('new_value',
            self.gf('django.db.models.fields.TextField')(db_column='new_value_',
                blank=True)),
            ('lexeme', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.Lexeme'], null=True, db_column='lexeme_id_',
                blank=True)),
            ('pattern', self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.Pattern'], null=True,
                db_column='pattern_id_', blank=True)),
            ('row_id',
            self.gf('django.db.models.fields.IntegerField')(db_column='id_')),
            ('operation',
            self.gf('django.db.models.fields.CharField')(max_length=120,
                db_column='operation_')),
            ('table_oid', self.gf('django.db.models.fields.IntegerField')(
                db_column='table_oid_')),
            ('column_ord', self.gf('django.db.models.fields.IntegerField')(
                db_column='ordinal_position_of_column_')),
            ('transaction_began',
            self.gf('django.db.models.fields.DateTimeField')(
                db_column='transaction_began_')),
        ))
        db.send_create_signal('dictionary', ['History'])

        # Adding model 'InputLexeme'
        db.create_table('dictionary_inputlexeme', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('entry',
            self.gf('django.db.models.fields.CharField')(max_length=64,
                db_index=True)),
        ))
        db.send_create_signal('dictionary', ['InputLexeme'])

        # Adding model 'InputForm'
        db.create_table('dictionary_inputform', (
            ('id',
            self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('input_lexeme',
            self.gf('django.db.models.fields.related.ForeignKey')(
                to=orm['dictionary.InputLexeme'])),
            ('form', self.gf('django.db.models.fields.CharField')(max_length=64,
                db_index=True)),
        ))
        db.send_create_signal('dictionary', ['InputForm'])


    def backwards(self, orm):
        # Removing unique constraint on 'SavedFilter', fields ['name', 'user']
        db.delete_unique('dictionary_savedfilter', ['name', 'user_id'])

        # Removing unique constraint on 'LexemeInflectionPattern', fields ['lexeme', 'index']
        db.delete_unique('odmieniasie', ['l_id', 'oind'])

        # Removing unique constraint on 'Ending', fields ['pattern', 'base_form_label', 'index']
        db.delete_unique('zakonczenia', ['w_id', 'efobaz', 'zind'])

        # Removing unique constraint on 'InflectionCharacteristic', fields ['entry', 'part_of_speech']
        db.delete_unique('charfle', ['charfl', 'pos'])

        # Removing unique constraint on 'Qualifier', fields ['label', 'vocabulary']
        db.delete_unique('kwalifikatory', ['kwal', 'slownik'])

        # Deleting model 'QualifierExclusionClass'
        db.delete_table('klasy_wykluczania')

        # Deleting model 'Qualifier'
        db.delete_table('kwalifikatory')

        # Deleting model 'Classification'
        db.delete_table('klasyfikacje')

        # Deleting model 'ClassificationValue'
        db.delete_table('wartosci_klasyfikacji')

        # Removing M2M table for field lexemes on 'ClassificationValue'
        db.delete_table('wartosci_klasyfikacji_lexemes')

        # Deleting model 'LexicalClass'
        db.delete_table('czescimowy')

        # Deleting model 'PartOfSpeech'
        db.delete_table('klasygramatyczne')

        # Deleting model 'BaseFormLabel'
        db.delete_table('efobazy')

        # Deleting model 'InflectionCharacteristic'
        db.delete_table('charfle')

        # Deleting model 'PatternType'
        db.delete_table('typywzorow')

        # Deleting model 'Pattern'
        db.delete_table('wzory')

        # Deleting model 'Ending'
        db.delete_table('zakonczenia')

        # Removing M2M table for field qualifiers on 'Ending'
        db.delete_table('kwalifikatory_zakonczen')

        # Deleting model 'Lexeme'
        db.delete_table('leksemy')

        # Removing M2M table for field qualifiers on 'Lexeme'
        db.delete_table('kwalifikatory_leksemow')

        # Deleting model 'LexemeInflectionPattern'
        db.delete_table('odmieniasie')

        # Removing M2M table for field qualifiers on 'LexemeInflectionPattern'
        db.delete_table('kwalifikatory_odmieniasiow')

        # Deleting model 'Vocabulary'
        db.delete_table('slowniki')

        # Removing M2M table for field managers on 'Vocabulary'
        db.delete_table('slowniki_managers')

        # Removing M2M table for field viewers on 'Vocabulary'
        db.delete_table('slowniki_viewers')

        # Removing M2M table for field editors on 'Vocabulary'
        db.delete_table('slowniki_editors')

        # Removing M2M table for field classifications on 'Vocabulary'
        db.delete_table('slowniki_classifications')

        # Deleting model 'LexemeAssociation'
        db.delete_table('leksemy_w_slownikach')

        # Deleting model 'CrossReferenceType'
        db.delete_table('typyodsylaczy')

        # Deleting model 'CrossReference'
        db.delete_table('odsylacze')

        # Deleting model 'Variant'
        db.delete_table('warianty')

        # Deleting model 'TableTemplate'
        db.delete_table('szablony_tabel')

        # Deleting model 'Cell'
        db.delete_table('klatki')

        # Deleting model 'TableCell'
        db.delete_table('komorki_tabel')

        # Deleting model 'TableHeader'
        db.delete_table('naglowki_tabel')

        # Deleting model 'ParadygmatyWSJP'
        db.delete_table('paradygmatywsjp')

        # Deleting model 'LexemeForm'
        db.delete_table('dictionary_lexemeform')

        # Deleting model 'SavedFilter'
        db.delete_table('dictionary_savedfilter')

        # Deleting model 'SavedExportData'
        db.delete_table('dictionary_savedexportdata')

        # Deleting model 'History'
        db.delete_table('history')

        # Deleting model 'InputLexeme'
        db.delete_table('dictionary_inputlexeme')

        # Deleting model 'InputForm'
        db.delete_table('dictionary_inputform')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [],
            {'unique': 'True', 'max_length': '80'}),
            'permissions': (
            'django.db.models.fields.related.ManyToManyField', [],
            {'to': "orm['auth.Permission']", 'symmetrical': 'False',
            'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {
            'ordering': "('content_type__app_label', 'content_type__model', 'codename')",
            'unique_together': "(('content_type', 'codename'),)",
            'object_name': 'Permission'},
            'codename': (
            'django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['contenttypes.ContentType']"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': (
            'django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [],
            {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [],
            {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [],
            {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [],
            {'to': "orm['auth.Group']", 'symmetrical': 'False',
            'blank': 'True'}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': (
            'django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': (
            'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': (
            'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [],
            {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [],
            {'max_length': '30', 'blank': 'True'}),
            'password': (
            'django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': (
            'django.db.models.fields.related.ManyToManyField', [],
            {'to': "orm['auth.Permission']", 'symmetrical': 'False',
            'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [],
            {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)",
            'unique_together': "(('app_label', 'model'),)",
            'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': (
            'django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': (
            'django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': (
            'django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'dictionary.baseformlabel': {
            'Meta': {'object_name': 'BaseFormLabel', 'db_table': "'efobazy'"},
            'entry': ('django.db.models.fields.CharField', [],
            {'max_length': '32', 'db_column': "'efobaz'", 'blank': 'True'}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'dictionary.cell': {
            'Meta': {'ordering': "['index']", 'object_name': 'Cell',
            'db_table': "'klatki'"},
            'base_form_label': (
            'django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.BaseFormLabel']", 'db_column': "'efobaz'"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'index': ('django.db.models.fields.IntegerField', [],
            {'db_column': "'kind'"}),
            'prefix': ('django.db.models.fields.CharField', [],
            {'max_length': '20', 'db_column': "'prefiks'", 'blank': 'True'}),
            'suffix': ('django.db.models.fields.CharField', [],
            {'max_length': '20', 'db_column': "'sufiks'", 'blank': 'True'}),
            'table_template': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.TableTemplate']", 'db_column': "'st_id'"}),
            'tag': ('django.db.models.fields.TextField', [],
            {'db_column': "'tag'", 'blank': 'True'})
        },
        'dictionary.classification': {
            'Meta': {'object_name': 'Classification',
            'db_table': "'klasyfikacje'"},
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [],
            {'unique': 'True', 'max_length': '64', 'db_column': "'nazwa'"})
        },
        'dictionary.classificationvalue': {
            'Meta': {'object_name': 'ClassificationValue',
            'db_table': "'wartosci_klasyfikacji'"},
            'classification': ('django.db.models.fields.related.ForeignKey', [],
            {'related_name': "'values'", 'db_column': "'klas_id'",
            'to': "orm['dictionary.Classification']"}),
            'deleted': ('django.db.models.fields.BooleanField', [],
            {'default': 'False', 'db_column': "'usunieta'"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [],
            {'unique': 'True', 'max_length': '64', 'db_column': "'nazwa'"}),
            'lexemes': ('django.db.models.fields.related.ManyToManyField', [],
            {'to': "orm['dictionary.Lexeme']", 'symmetrical': 'False',
            'blank': 'True'}),
            'parent_node': ('django.db.models.fields.related.ForeignKey', [],
            {'blank': 'True', 'related_name': "'child_nodes'", 'null': 'True',
            'db_column': "'rodzic'",
            'to': "orm['dictionary.ClassificationValue']"})
        },
        'dictionary.crossreference': {
            'Meta': {'object_name': 'CrossReference',
            'db_table': "'odsylacze'"},
            'from_lexeme': ('django.db.models.fields.related.ForeignKey', [],
            {'related_name': "'refs_to'", 'db_column': "'l_id_od'",
            'to': "orm['dictionary.Lexeme']"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'to_lexeme': ('django.db.models.fields.related.ForeignKey', [],
            {'related_name': "'refs_from'", 'db_column': "'l_id_do'",
            'to': "orm['dictionary.Lexeme']"}),
            'type': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.CrossReferenceType']",
            'db_column': "'typods_id'"})
        },
        'dictionary.crossreferencetype': {
            'Meta': {'object_name': 'CrossReferenceType',
            'db_table': "'typyodsylaczy'"},
            'desc': ('django.db.models.fields.CharField', [],
            {'max_length': '40', 'db_column': "'naglowek'"}),
            'from_pos': ('django.db.models.fields.related.ForeignKey', [],
            {'related_name': "'crtype_to'", 'db_column': "'pos1'",
            'to': "orm['dictionary.PartOfSpeech']"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'index': ('django.db.models.fields.IntegerField', [],
            {'db_column': "'kolejnosc'"}),
            'symbol': ('django.db.models.fields.CharField', [],
            {'max_length': '10', 'db_column': "'typods'"}),
            'to_pos': ('django.db.models.fields.related.ForeignKey', [],
            {'related_name': "'crtype_from'", 'db_column': "'pos2'",
            'to': "orm['dictionary.PartOfSpeech']"})
        },
        'dictionary.ending': {
            'Meta': {'ordering': "['index']",
            'unique_together': "(('pattern', 'base_form_label', 'index'),)",
            'object_name': 'Ending', 'db_table': "'zakonczenia'"},
            'base_form_label': (
            'django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.BaseFormLabel']", 'db_column': "'efobaz'"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'index': ('django.db.models.fields.IntegerField', [],
            {'db_column': "'zind'"}),
            'pattern': ('django.db.models.fields.related.ForeignKey', [],
            {'related_name': "'endings'", 'db_column': "'w_id'",
            'to': "orm['dictionary.Pattern']"}),
            'qualifiers': (
            'django.db.models.fields.related.ManyToManyField', [],
            {'to': "orm['dictionary.Qualifier']", 'symmetrical': 'False',
            'db_table': "'kwalifikatory_zakonczen'", 'blank': 'True'}),
            'string': ('django.db.models.fields.CharField', [],
            {'max_length': '16', 'db_column': "'zak'", 'blank': 'True'})
        },
        'dictionary.history': {
            'Meta': {'object_name': 'History', 'db_table': "'history'"},
            'column_name': ('django.db.models.fields.CharField', [],
            {'max_length': '120', 'db_column': "'column_name_'",
            'blank': 'True'}),
            'column_ord': ('django.db.models.fields.IntegerField', [],
            {'db_column': "'ordinal_position_of_column_'"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lexeme': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.Lexeme']", 'null': 'True',
            'db_column': "'lexeme_id_'", 'blank': 'True'}),
            'new_value': ('django.db.models.fields.TextField', [],
            {'db_column': "'new_value_'", 'blank': 'True'}),
            'old_value': ('django.db.models.fields.TextField', [],
            {'db_column': "'old_value_'", 'blank': 'True'}),
            'operation': ('django.db.models.fields.CharField', [],
            {'max_length': '120', 'db_column': "'operation_'"}),
            'pattern': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.Pattern']", 'null': 'True',
            'db_column': "'pattern_id_'", 'blank': 'True'}),
            'row_id': (
            'django.db.models.fields.IntegerField', [], {'db_column': "'id_'"}),
            'table_name': ('django.db.models.fields.CharField', [],
            {'max_length': '120', 'db_column': "'table_name_'"}),
            'table_oid': ('django.db.models.fields.IntegerField', [],
            {'db_column': "'table_oid_'"}),
            'timestamp': ('django.db.models.fields.DateTimeField', [],
            {'db_column': "'timestamp_'"}),
            'transaction_began': ('django.db.models.fields.DateTimeField', [],
            {'db_column': "'transaction_began_'"}),
            'user': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['auth.User']", 'db_column': "'user_id_'"})
        },
        'dictionary.inflectioncharacteristic': {
            'Meta': {'unique_together': "(['entry', 'part_of_speech'],)",
            'object_name': 'InflectionCharacteristic', 'db_table': "'charfle'"},
            'basic_form_label': (
            'django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.BaseFormLabel']", 'db_column': "'efobaz'"}),
            'entry': ('django.db.models.fields.CharField', [],
            {'max_length': '16', 'db_column': "'charfl'", 'blank': 'True'}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'part_of_speech': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.PartOfSpeech']", 'db_column': "'pos'"})
        },
        'dictionary.inputform': {
            'Meta': {'object_name': 'InputForm'},
            'form': ('django.db.models.fields.CharField', [],
            {'max_length': '64', 'db_index': 'True'}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'input_lexeme': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.InputLexeme']"})
        },
        'dictionary.inputlexeme': {
            'Meta': {'object_name': 'InputLexeme'},
            'entry': ('django.db.models.fields.CharField', [],
            {'max_length': '64', 'db_index': 'True'}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'dictionary.lexeme': {
            'Meta': {'object_name': 'Lexeme', 'db_table': "'leksemy'"},
            'comment': ('django.db.models.fields.TextField', [],
            {'db_column': "'komentarz'", 'blank': 'True'}),
            'deleted': ('django.db.models.fields.BooleanField', [],
            {'default': 'False', 'db_column': "'usuniety'"}),
            'entry': ('django.db.models.fields.CharField', [],
            {'db_index': 'True', 'max_length': '64', 'db_column': "'haslo'",
            'blank': 'True'}),
            'entry_suffix': ('django.db.models.fields.CharField', [],
            {'max_length': '16', 'db_column': "'haslosuf'", 'blank': 'True'}),
            'gloss': ('django.db.models.fields.TextField', [],
            {'db_column': "'glosa'", 'blank': 'True'}),
            'homonym_number': (
            'django.db.models.fields.IntegerField', [], {'db_column': "'hom'"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_modified': ('django.db.models.fields.DateTimeField', [],
            {'auto_now': 'True', 'db_column': "'data_modyfikacji'",
            'blank': 'True'}),
            'owner_vocabulary': (
            'django.db.models.fields.related.ForeignKey', [],
            {'related_name': "'owned_lexemes'", 'db_column': "'slownik'",
            'to': "orm['dictionary.Vocabulary']"}),
            'part_of_speech': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.PartOfSpeech']", 'db_column': "'pos'"}),
            'patterns': ('django.db.models.fields.related.ManyToManyField', [],
            {'to': "orm['dictionary.Pattern']",
            'through': "orm['dictionary.LexemeInflectionPattern']",
            'symmetrical': 'False'}),
            'qualifiers': (
            'django.db.models.fields.related.ManyToManyField', [],
            {'to': "orm['dictionary.Qualifier']", 'symmetrical': 'False',
            'db_table': "'kwalifikatory_leksemow'", 'blank': 'True'}),
            'responsible': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['auth.User']", 'null': 'True',
            'db_column': "'odpowiedzialny'", 'blank': 'True'}),
            'source': ('django.db.models.fields.CharField', [],
            {'max_length': '32', 'db_column': "'zrodlo'", 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [],
            {'max_length': '8', 'db_column': "'status'"})
        },
        'dictionary.lexemeassociation': {
            'Meta': {'object_name': 'LexemeAssociation',
            'db_table': "'leksemy_w_slownikach'"},
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lexeme': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.Lexeme']", 'db_column': "'l_id'"}),
            'vocabulary': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.Vocabulary']", 'db_column': "'slownik'"})
        },
        'dictionary.lexemeform': {
            'Meta': {'object_name': 'LexemeForm'},
            'form': ('django.db.models.fields.CharField', [],
            {'max_length': '128', 'db_index': 'True'}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lexeme': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.Lexeme']"})
        },
        'dictionary.lexemeinflectionpattern': {
            'Meta': {'ordering': "['index']",
            'unique_together': "(('lexeme', 'index'),)",
            'object_name': 'LexemeInflectionPattern',
            'db_table': "'odmieniasie'"},
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'index': ('django.db.models.fields.IntegerField', [],
            {'db_column': "'oind'"}),
            'inflection_characteristic': (
            'django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.InflectionCharacteristic']",
            'db_column': "'charfl'"}),
            'lexeme': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.Lexeme']", 'db_column': "'l_id'"}),
            'pattern': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.Pattern']", 'db_column': "'w_id'"}),
            'qualifiers': (
            'django.db.models.fields.related.ManyToManyField', [],
            {'to': "orm['dictionary.Qualifier']", 'symmetrical': 'False',
            'db_table': "'kwalifikatory_odmieniasiow'", 'blank': 'True'}),
            'root': ('django.db.models.fields.CharField', [],
            {'max_length': '64', 'db_column': "'rdzen'"})
        },
        'dictionary.lexicalclass': {
            'Meta': {'object_name': 'LexicalClass', 'db_table': "'czescimowy'"},
            'symbol': ('django.db.models.fields.CharField', [],
            {'max_length': '16', 'primary_key': 'True', 'db_column': "'czm'"})
        },
        'dictionary.paradygmatywsjp': {
            'Meta': {'object_name': 'ParadygmatyWSJP',
            'db_table': "'paradygmatywsjp'"},
            'charfl': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.InflectionCharacteristic']",
            'db_column': "'charfl'"}),
            'col': ('django.db.models.fields.IntegerField', [], {}),
            'colspan': ('django.db.models.fields.IntegerField', [], {}),
            'efobaz': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.BaseFormLabel']", 'db_column': "'efobaz'"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'kskl': ('django.db.models.fields.IntegerField', [], {}),
            'morf': ('django.db.models.fields.TextField', [], {}),
            'podparad': ('django.db.models.fields.CharField', [],
            {'max_length': '4', 'blank': 'True'}),
            'pref': ('django.db.models.fields.CharField', [],
            {'max_length': '20', 'blank': 'True'}),
            'row': ('django.db.models.fields.IntegerField', [], {}),
            'rowspan': ('django.db.models.fields.IntegerField', [], {}),
            'suf': ('django.db.models.fields.CharField', [],
            {'max_length': '20', 'blank': 'True'}),
            'typr': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.PatternType']", 'db_column': "'typr'"}),
            'wariant': (
            'django.db.models.fields.CharField', [], {'max_length': '4'})
        },
        'dictionary.partofspeech': {
            'Meta': {'ordering': "['symbol']", 'object_name': 'PartOfSpeech',
            'db_table': "'klasygramatyczne'"},
            'lexical_class': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.LexicalClass']", 'db_column': "'czm'"}),
            'symbol': ('django.db.models.fields.CharField', [],
            {'max_length': '16', 'primary_key': 'True', 'db_column': "'pos'"})
        },
        'dictionary.pattern': {
            'Meta': {'ordering': "['name']", 'object_name': 'Pattern',
            'db_table': "'wzory'"},
            'basic_form_ending': ('django.db.models.fields.CharField', [],
            {'max_length': '32', 'db_column': "'zakp'", 'blank': 'True'}),
            'comment': ('django.db.models.fields.TextField', [],
            {'db_column': "'komentarz'", 'blank': 'True'}),
            'example': ('django.db.models.fields.CharField', [],
            {'max_length': '64', 'db_column': "'przyklad'"}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [],
            {'unique': 'True', 'max_length': '32', 'db_column': "'w_id'"}),
            'status': (
            'django.db.models.fields.CharField', [], {'max_length': '8'}),
            'type': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.PatternType']", 'db_column': "'typ'"})
        },
        'dictionary.patterntype': {
            'Meta': {'object_name': 'PatternType', 'db_table': "'typywzorow'"},
            'entry': ('django.db.models.fields.CharField', [],
            {'max_length': '32', 'db_column': "'wtyp'", 'blank': 'True'}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lexical_class': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.LexicalClass']", 'db_column': "'czm'"})
        },
        'dictionary.qualifier': {
            'Meta': {'ordering': "['label']",
            'unique_together': "(['label', 'vocabulary'],)",
            'object_name': 'Qualifier', 'db_table': "'kwalifikatory'"},
            'deleted': ('django.db.models.fields.BooleanField', [],
            {'default': 'False', 'db_column': "'usuniety'"}),
            'exclusion_class': (
            'django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.QualifierExclusionClass']", 'null': 'True',
            'db_column': "'klasa'", 'blank': 'True'}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [],
            {'max_length': '64', 'db_column': "'kwal'"}),
            'vocabulary': ('django.db.models.fields.related.ForeignKey', [],
            {'related_name': "'qualifiers'", 'db_column': "'slownik'",
            'to': "orm['dictionary.Vocabulary']"})
        },
        'dictionary.qualifierexclusionclass': {
            'Meta': {'object_name': 'QualifierExclusionClass',
            'db_table': "'klasy_wykluczania'"},
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [],
            {'unique': 'True', 'max_length': '64', 'db_column': "'nazwa'"}),
            'vocabulary': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.Vocabulary']", 'db_column': "'slownik'"})
        },
        'dictionary.savedexportdata': {
            'Meta': {'object_name': 'SavedExportData'},
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [],
            {'unique': 'True', 'max_length': '64'}),
            'serialized_data': ('django.db.models.fields.TextField', [], {})
        },
        'dictionary.savedfilter': {
            'Meta': {'unique_together': "(('name', 'user'),)",
            'object_name': 'SavedFilter'},
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': (
            'django.db.models.fields.CharField', [], {'max_length': '64'}),
            'serialized_filter': ('django.db.models.fields.TextField', [], {}),
            'super': (
            'django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'user': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['auth.User']"})
        },
        'dictionary.tablecell': {
            'Meta': {'object_name': 'TableCell', 'db_table': "'komorki_tabel'"},
            'cell': ('django.db.models.fields.related.OneToOneField', [],
            {'to': "orm['dictionary.Cell']", 'unique': 'True',
            'db_column': "'k_id'"}),
            'col': ('django.db.models.fields.IntegerField', [], {}),
            'colspan': ('django.db.models.fields.IntegerField', [], {}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'row': ('django.db.models.fields.IntegerField', [], {}),
            'rowspan': ('django.db.models.fields.IntegerField', [], {})
        },
        'dictionary.tableheader': {
            'Meta': {'object_name': 'TableHeader',
            'db_table': "'naglowki_tabel'"},
            'col': ('django.db.models.fields.IntegerField', [], {}),
            'colspan': ('django.db.models.fields.IntegerField', [], {}),
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [],
            {'max_length': '64', 'db_column': "'nagl'", 'blank': 'True'}),
            'row': ('django.db.models.fields.IntegerField', [], {}),
            'row_header': ('django.db.models.fields.BooleanField', [],
            {'default': 'False', 'db_column': "'wierszowy'"}),
            'rowspan': ('django.db.models.fields.IntegerField', [], {}),
            'table_template': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.TableTemplate']", 'db_column': "'st_id'"})
        },
        'dictionary.tabletemplate': {
            'Meta': {'object_name': 'TableTemplate',
            'db_table': "'szablony_tabel'"},
            'id': (
            'django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'inflection_characteristic': (
            'django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.InflectionCharacteristic']",
            'db_column': "'charfl'"}),
            'pattern_type': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.PatternType']", 'db_column': "'wtyp'"}),
            'variant': ('django.db.models.fields.related.ForeignKey', [],
            {'to': "orm['dictionary.Variant']", 'db_column': "'wariant'"})
        },
        'dictionary.variant': {
            'Meta': {'object_name': 'Variant', 'db_table': "'warianty'"},
            'id': ('django.db.models.fields.CharField', [],
            {'max_length': '32', 'primary_key': 'True',
            'db_column': "'wariant'"})
        },
        'dictionary.vocabulary': {
            'Meta': {'object_name': 'Vocabulary', 'db_table': "'slowniki'"},
            'classifications': (
            'django.db.models.fields.related.ManyToManyField', [],
            {'symmetrical': 'False', 'related_name': "'vocabularies'",
            'blank': 'True', 'to': "orm['dictionary.Classification']"}),
            'editors': ('django.db.models.fields.related.ManyToManyField', [],
            {'symmetrical': 'False', 'related_name': "'editable_vocabularies'",
            'blank': 'True', 'to': "orm['auth.User']"}),
            'id': ('django.db.models.fields.CharField', [],
            {'max_length': '64', 'primary_key': 'True',
            'db_column': "'slownik'"}),
            'lexemes': ('django.db.models.fields.related.ManyToManyField', [],
            {'symmetrical': 'False', 'related_name': "'vocabularies'",
            'blank': 'True', 'through': "orm['dictionary.LexemeAssociation']",
            'to': "orm['dictionary.Lexeme']"}),
            'managers': ('django.db.models.fields.related.ManyToManyField', [],
            {'symmetrical': 'False', 'related_name': "'managed_vocabularies'",
            'blank': 'True', 'to': "orm['auth.User']"}),
            'viewers': ('django.db.models.fields.related.ManyToManyField', [],
            {'symmetrical': 'False', 'related_name': "'visible_vocabularies'",
            'blank': 'True', 'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['dictionary']