/* global $dj, getBusyOverlay, common, gettext */

var edit = {
    changed: false,
    created: false,

    show_changed: function() {
        "use strict";
        if (!edit.changed) {
            edit.changed = true;
            $('.' + edit.form_submit_class).button('enable');
            $('.' + edit.form_cancel_class).button('enable');
            $('#edit-tab').html(
                '<strong>' + gettext("Edycja") + ' (' + gettext("niezapisane") +
                ')</strong>');
            $(window).on('beforeunload', function () {
                return gettext("Są niezapisane zmiany.");
            });
        }
    },

    hide_changed: function() {
        "use strict";
        if (edit.changed) {
            edit.changed = false;
            $('#edit-tab').html(gettext("Edycja"));
            $(window).off('beforeunload');
        }
    },

    qualifier_select: function(jq) {
        "use strict";
        jq.multiSelect({
            selectedText: selected_qualifiers_text,//'# kwalifikatorów',
            create: function () {
                var select = $(this);
                var selected = select.val();
                if (selected) {
                    $.each(selected, function (i, value) {
                        toggle_excluded(select, value, false);
                    });
                }
            },
            click: function (event, ui) {
                toggle_excluded(this, ui.value, !ui.checked);
            }
        });
        jq.next().width('100px');
    },

    get_id: function() {
        "use strict";
        return edit.active_id;
    },

    // busy

    busy_ctrl: null,

    busy_on: function() {
        "use strict";
        if (!edit.busy_ctrl) {
            edit.busy_ctrl = getBusyOverlay(
                $('#right')[0],
                {color: 'black', opacity: 0.2},
                {size: 30});
        }
    },

    busy_off: function() {
        "use strict";
        if (edit.busy_ctrl) edit.busy_ctrl.remove();
        edit.busy_ctrl = null;
    },

    confirm_discard: function() {
        "use strict";
        return window.confirm(
            gettext("Czy chcesz porzucić niezapisane zmiany?"));
    }
};

function selected_qualifiers_text(num, total, checked) {
    "use strict";
    if (num >= 4)
        return num + ' ' + gettext("kwalifikatorów");
    var grouped = {};
    $.each(checked, function (i, elem) {
        var id = elem.value;
        var vocab = $dj.vocabs[id];
        if (grouped[vocab] === undefined)
            grouped[vocab] = [];
        grouped[vocab].push(elem.title);
    });
    var vocab_table = [];
    var owner = $('#owner-vocabulary').text();
    if (grouped[owner])
        vocab_table.push(owner + ': ' + grouped[owner].join(', '));
    $.each(grouped, function (vocab, qualifiers) {
        if (vocab !== owner)
            vocab_table.push(vocab + ': ' + qualifiers.join(', '));
    });
    return vocab_table.join('; ');
}

var reverse_exclusion_classes = {};

$.each($dj.exclusion_classes, function (ec, qualifiers) {
    "use strict";
    $.each(qualifiers, function (i, q_id) {
        reverse_exclusion_classes[q_id] = ec;
    });
});

function toggle_excluded(select, value, enable) {
    "use strict";
    var ec = reverse_exclusion_classes[Number(value)],
        values = $dj.exclusion_classes[ec];
    if (values)
        $.each(values, function (i, v) {
            if (v !== Number(value))
                common.multiselect_toggle(select, v, enable);
        });
}

$(function() {
    "use strict";
    $(document).on(
        'submit', '#' + edit.form_id, function() {
            $(this).find('button[type=submit]').button('disable');
            var this_form = $(this);
            var form_data = this_form.serializeArray();
            edit.form_submit(form_data);
            return false;
        });
    $(document).on('click', '.' + edit.form_cancel_class, function () {
        $.ajaxJSON({
            method: 'get',
            dest: $('#edit'),
            url: $dj.ajax_edit_form,
            data: {id: edit.get_id()},
            callback: edit.form_init
        });
    });
    var keyup = function () {
        if (this.value !== this.defaultValue) {
            edit.show_changed();
        }
    };
    $(document).on('keyup', '#' + edit.form_id + ' input', keyup);
    $(document).on('keyup', '#' + edit.form_id + ' textarea', keyup);
    $(document).on('change', '#' + edit.form_id + ' select', keyup);
    $(document).on(
        'change', '#' + edit.form_id + ' input[type=checkbox]',
        edit.show_changed);
    $(document).on('click', '.' + edit.form_cancel_class, function () {
        $.ajaxJSON({
            method: 'get',
            dest: $('#edit'),
            url: $dj.ajax_edit_form,
            data: {id: edit.get_id()},
            callback: edit.form_init
        });
    });
    edit.init_form_widgets();
});