# -*- coding: utf-8 -*-
import re

from django import template
from django.utils.safestring import mark_safe

from dictionary.models import LexemeAttribute
from history.pattern_history import pattern_table_from_row
from history.lexeme_history import lexeme_table_from_row

register = template.Library()


# potrzebne do rekurencji
@register.inclusion_tag('classification_value_tree.html')
def value_tree(tree):
    return {'value_tree': tree}


@register.filter
def lexeme_table(row):
    return lexeme_table_from_row(row)


@register.filter
def pattern_table(row):
    return pattern_table_from_row(row)


@register.filter
def attribute(lexeme, attr_name):
    attr = LexemeAttribute.objects.get(name=attr_name)
    if attr.multiple:
        return lexeme.attribute_value(attr).values_list(
            'display_value', flat=True)
    elif not attr.closed:
        attr_val = lexeme.attribute_value(attr)
        return attr_val.value if attr_val else ''
    else:
        attr_val = lexeme.attribute_value(attr)
        return attr_val.display_value if attr_val else ''


@register.filter
def sgjp_markup(text):
    text = re.sub(r'\\ods{([^}]*)}', r'<strong>\1</strong>', text)
    text = re.sub(r'\\pim{([^}]*)}', r'<em>\1</em>', text)
    return mark_safe(text)