# -*- coding: utf-8 -*-
# Django settings for lexeme_forge project.

import os.path
import database_data
from django.utils.translation import ugettext_lazy as _

BASE_DIR = os.path.dirname(__file__)

SITE_PREFIX = ''

DEBUG = True
TEMPLATE_DEBUG = DEBUG
SHOW_SQL_QUERIES = False
SQL_STACKTRACE = False

INTERNAL_IPS = ('127.0.0.1',)

ALLOWED_HOSTS = ('localhost',)

# CSRF_COOKIE_PATH = SITE_PREFIX
# SESSION_COOKIE_PATH = SITE_PREFIX
# LANGUAGE_COOKIE_PATH = SITE_PREFIX

ADMINS = (
    ('Jan Szejko', 'jan.szejko@ipipan.waw.pl'),
)

MANAGERS = ADMINS

DATABASES = database_data.DATABASES

DATABASE_ENGINE = 'django.db.backends.postgresql_psycopg2'

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'TIMEOUT': 7200,
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/Warsaw'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'pl'

LANGUAGES = [
    ('en', _('English')),
    ('pl', _('Polish')),
]

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

LOCALE_PATHS = (
    os.path.join(BASE_DIR, "accounts/locale"),
    os.path.join(BASE_DIR, "common/locale"),
    os.path.join(BASE_DIR, "dictionary/locale"),
    os.path.join(BASE_DIR, "export/locale"),
    os.path.join(BASE_DIR, "history/locale"),
    os.path.join(BASE_DIR, "management/locale"),
    os.path.join(BASE_DIR, "paginer/locale"),
    os.path.join(BASE_DIR, "patterns/locale"),
)

STATIC_ROOT = os.path.join(BASE_DIR, 'static')

STATIC_URL = SITE_PREFIX + '/static/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = SITE_PREFIX + '/admin_media/'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.app_directories.Loader',
    # 'django.template.loaders.filesystem.Loader',
    # 'django.template.loaders.eggs.Loader',
)

# TEMPLATE_DIRS = ()

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.transaction.TransactionMiddleware',
    # 'debug_toolbar.middleware.DebugToolbarMiddleware',
    'common.middleware.MyMiddleware',
)

ROOT_URLCONF = 'urls_main'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # dodatki
    'registration',
    'django_extensions',
    'south',
    # 'debug_toolbar',
    'sqlstacktrace',
    # aplikacje projektu
    'dictionary',
    'patterns',
    'management',
    'export',
    'accounts',
    'common',
    'history',
    'paginer',
    'django.contrib.admin',  # ze względu na lokalizację
)

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

# Account settings
ACCOUNT_ACTIVATION_DAYS = 10
LOGIN_URL = SITE_PREFIX + '/accounts/login/'
LOGIN_REDIRECT_URL = SITE_PREFIX + '/'

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler',
            'include_html': True,
        },
        'null': {
            'level': 'DEBUG',
            'class': 'logging.NullHandler',
        },
    },
    'loggers': {
        'django.security.DisallowedHost': {
            'handlers': ['null'],
            'propagate': False,
        },
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

SECRET_FILE = os.path.join(BASE_DIR, 'secret.txt')
try:
    SECRET_KEY = open(SECRET_FILE).read().strip()
except IOError:
    try:
        import random
        SECRET_KEY = ''.join([
            random.SystemRandom().choice(
                'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)')
            for i in range(50)])
        secret = file(SECRET_FILE, 'w')
        secret.write(SECRET_KEY)
        secret.close()
    except IOError:
        raise Exception(
            'Please create a %s file with random characters '
            'to generate your secret key!' % SECRET_FILE)
